//##########################################################################
//#                                                                        #
//#        Copyright (C) 2011 by Beijing IRTOUCHSYSTEMS Co., Ltd          #
//#                                                                        #
//#       IRTOUCH Optical USB Touchscreen                                  #    
//#                                                                        #
//#       Create by Smart Zhu                                              #
//#                                                                        #
//#       Email: zh@unitop.cc                                              #
//#                                                                        #
//##########################################################################

#ifndef IRTUSB_H
#define IRTUSB_H

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/usb.h>
#include <linux/input.h>
#include <linux/usb/input.h>
#include <linux/hid.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <asm/uaccess.h> 

#define MAJOR_DEVICE_NUMBER 47
#define MINOR_DEVICE_NUMBER 192


#define USB_IRTUSB_VENDOR_ID  0x6615
#define USB_IRTUSB_PRODUCT_ID 0x0C20


#define COMMAND_REPORTID 0xAA
#define DATA_COORDINATE 0x01

#define SET_COMMAND 0x01
#define GET_COMMAND 0x02

#define MAX_LENGTH 64
#define COMMAND_RETRY_COUNT 5
#define IMAGE_WIDTH 640

#define MODE_COORDINATE 0x00
#define MODE_CONFIG 0x01

#define PRIMARYID 	0x00
#define SECONDARYID 	0x01

#define COMMAND_SET_MODE 0x71
#define COMMAND_GET_CAMERAPARA 0x10
#define COMMAND_GET_CALIBPARA 0x14
#define COMMAND_SET_CALIBPARA 0x15
#define COMMAND_GET_PRODUCTPARA 0x30
#define COMMAND_SET_PRODUCTPARA 0x31

//calibration
#define CTLCODE_COORDINATE 0xc0
#define CTLCODE_SET_CALIB_PARA_X 0xc1
#define CTLCODE_SET_CALIB_PARA_Y 0xc2
#define CTLCODE_CALIB_START 0xc3

#define CTLCODE_SCREEN_PROP 0xc4
#define CTLCODE_PRIMARY_CAMERA 0xc5
#define CTLCODE_SECONDLY_CAMERA 0xc6
#define CTLCODE_DEVICE_CONFIG 0xc7
#define CTLCODE_CALIB_PARA_X 0xc8
#define CTLCODE_CALIB_PARA_Y 0xc9
#define CTLCODE_UPDATE_PARAM 0xca

#define START_CALIB 0Xf0
#define UPDATE_PARAM 0Xf1

#define TOUCH_DEVICE_NAME "irtouch"

#pragma pack(1)
typedef struct _OptPoint{
	int	x;
	int	y;
	int	radius;
	bool	isExist;
}OptPoint, *POptPoint;

typedef struct _CalibPara
{
	float	A03;
	float	A12;
	float	A21;
	float	A30;
	float	A02;
	float	A11;
	float	A20;
	float	A01;
	float	A10;
	float	A00;
	float	reserve0;
	float	reserve1;
	float	reserve2;
	float	reserve3;
	unsigned char	reserve4;
	unsigned char	reserve5;
}CalibPara, *PCalibPara;

typedef struct _OptCamProps
{
	unsigned char Pos;	//'L' or 'R'
	unsigned char ID;	//'0'='Primary' or '1'='Secondary'
	int 	lowEdge;
	int 	highEdge;
	float 	a0;		//default = 0;
	float 	a1;		//default = 1.15;
	float 	a2;		//default = 0;
	float 	a3;		//default = 0;
	float 	a4;		//default = 0;
	float 	a5;		//default = 0;
	float 	a6;		//default = 0;
	float 	a7;		//default = 0;
	float 	b0;		//default = 0;
	float 	b1;		//default = 1;
	float 	reserve0;
	float 	reserve1;
}OptCamProps, *POptCamProps;

typedef struct _OptScrProps
{
	unsigned char pos;
	unsigned char s1;
	int 	width;		//unit:mm/s
	int 	height;		//unit:mm/s
	int 	fps;
	int 	maxSpeed;	//unit:mm/s
	int 	camOffsetY;	//unit:mm
	int 	camLOffsetX;	//unit:mm
	int 	camROffsetX;	//unit:mm	
	int 	velocityRefCount;    
	float 	veloctiyWeighing;    
	float 	reserve0;
	float 	reserve1;
	float 	reserve2;
	float 	reserve3;
	float 	reserve4;
}OptScrProps, *POptScrProps;

struct optical_raw_data{
	union
	{
		struct
		{
			unsigned char ReportID;
			unsigned char PacketType;
			unsigned char CmdType;
			unsigned char Length;
			unsigned char DeviceID;
			unsigned char Data[58];
			unsigned char crc;
		}CmdPacket;
		unsigned char RawInput[64];
	};
};

struct device_config{
	unsigned char DeviceID;
	unsigned char MonitorID;
	unsigned short MinX;
	unsigned short MinY;
	unsigned short MaxX;
	unsigned short MaxY;
	unsigned char IsCalibrated;
	long OffX;
	long OffY;
	unsigned short ResolutionX;
	unsigned short ResolutionY;
	unsigned short PrimaryResolutionX;
	unsigned short PrimaryResolutionY;
	unsigned char Reserved[31];
};

struct driver_config{
	unsigned char Mode;
	unsigned char EnableTouch;
	unsigned char EnableRightClick;
	unsigned char ClibrateType;
	unsigned char EnableDoubleClick;
	unsigned char HideCursor;
	unsigned char EnableBeep;
	unsigned char BeepType;
	unsigned char BeepMode;
	unsigned char IsCalibrating;
	unsigned char BeepFrequency;
	unsigned char BeepDuration;
	unsigned char Smoothness;
	unsigned char RightClickTime;
	unsigned char RightClickArea;
	unsigned char DoubleClickTime;
	unsigned char DoubleClickArea;
	unsigned char Reserved[20];
};

struct points
{
	int	x;
	int	y;
};
#pragma pack()

struct device_context{
	OptScrProps ScreenProps;
	OptCamProps PrimaryCamProps;
	OptCamProps SecondaryCamProps;
	CalibPara calibX;
	CalibPara calibY;
	OptPoint Points[2];

	struct points points;
	struct driver_config drvConfig;
	struct device_config devConfig;

	bool bSendDown[2];
	bool bCalibX;
	bool bCalibY;
	bool bStartCalib;
	bool bCalibrated;
	bool bUpdateParam;
};

struct irtouchusb{
	char name[128];
	char phys[64];

	struct usb_device *udev;
	struct input_dev *dev;
	struct device_context * context;
	
	char inbuf[64];
};

#endif //IRTUSB_H
